"use strict";(self.webpackChunkeodag_labextension=self.webpackChunkeodag_labextension||[]).push([[157,504],{3157:(e,t,n)=>{n.r(t),n.d(t,{NonceProvider:()=>d,components:()=>h.c,createFilter:()=>o.c,default:()=>v,defaultTheme:()=>o.d,mergeStyles:()=>o.m,useStateManager:()=>r.u});var r=n(4766),a=n(2122),c=n(6271),o=n(5037),i=n(6610),u=n(5991),s=n(379),h=n(8922),l=n(2443),f=n(7275),m=n(845),p=(n(4456),(0,c.forwardRef)((function(e,t){var n=(0,r.u)(e);return c.createElement(o.S,(0,a.Z)({ref:t},n))}))),d=function(e){(0,s.Z)(n,e);var t=(0,h._)(n);function n(e){var r;return(0,i.Z)(this,n),(r=t.call(this,e)).createEmotionCache=function(e,t){return(0,f.Z)({nonce:e,key:t})},r.createEmotionCache=(0,m.Z)(r.createEmotionCache),r}return(0,u.Z)(n,[{key:"render",value:function(){var e=this.createEmotionCache(this.props.nonce,this.props.cacheKey);return c.createElement(l.C,{value:e},this.props.children)}}]),n}(c.Component);const v=p}}]);