.result-table {
  width: 100%;
  table-layout: fixed;
  border-collapse: collapse;
}
.result-table tr {
  border-bottom: 1px solid rgba(180, 180, 180, 0.719);
}
.result-table tr:hover {
  background-color: rgba(180, 180, 180, 0.747);
}
.result-table td {
  width: 50%;
  padding: 18px 20px;
  font-size: 15px;
  border-bottom: 1px solid rgba(180, 180, 180, 0.719);
}
#endDate-error-text {
  color: red;
  margin-top: -3px;
}
